@@ -1,5 +1,5 @@
-/* stringprep.c		Core stringprep implementation.
- * Copyright (C) 2002, 2003  Simon Josefsson
+/* stringprep.c --- Core stringprep implementation.
+ * Copyright (C) 2002, 2003, 2004  Simon Josefsson
  *
  * This file is part of GNU Libidn.
  *
@@ -424,8 +424,7 @@ stringprep (char *in,
 int
 stringprep_profile (const char *in,
 		    char **out,
-		    const char *profile,
-		    Stringprep_profile_flags flags)
+		    const char *profile, Stringprep_profile_flags flags)
 {
   const Stringprep_profiles *p;
   char *str = NULL;
@@ -473,18 +472,26 @@ stringprep_profile (const char *in,
  *
  * The library contains a generic Stringprep implementation that does
  * Unicode 3.2 NFKC normalization, mapping and prohibitation of
- * characters, and bidirectional character handling.  Profiles for iSCSI,
- * Kerberos 5, Nameprep, SASL and XMPP are included.  Punycode and ASCII
- * Compatible Encoding (ACE) via IDNA are supported.
+ * characters, and bidirectional character handling.  Profiles for
+ * Nameprep, iSCSI, SASL and XMPP are included.  Punycode and ASCII
+ * Compatible Encoding (ACE) via IDNA are supported.  A mechanism to
+ * define Top-Level Domain (TLD) specific validation tables, and to
+ * compare strings against those tables, is included.  Default tables
+ * for some TLDs are also included.
  *
- * The Stringprep API consists of two main functions, one for converting
- * data from the system's native representation into UTF-8, and one
- * function to perform the Stringprep processing.  Adding a new
- * Stringprep profile for your application within the API is
- * straightforward.  The Punycode API consists of one encoding function
- * and one decoding function.  The IDNA API consists of the ToASCII and
- * ToUnicode functions, as well as an high-level interface for converting
- * entire domain names to and from the ACE encoded form.
+ * The Stringprep API consists of two main functions, one for
+ * converting data from the system's native representation into UTF-8,
+ * and one function to perform the Stringprep processing.  Adding a
+ * new Stringprep profile for your application within the API is
+ * straightforward.  The Punycode API consists of one encoding
+ * function and one decoding function.  The IDNA API consists of the
+ * ToASCII and ToUnicode functions, as well as an high-level interface
+ * for converting entire domain names to and from the ACE encoded
+ * form.  The TLD API consists of one set of functions to extract the
+ * TLD name from a domain string, one set of functions to locate the
+ * proper TLD table to use based on the TLD name, and core functions
+ * to validate a string against a TLD table, and some utility wrappers
+ * to perform all the steps in one call.
  *
  * The library is used by, e.g., GNU SASL and Shishi to process user
  * names and passwords.  Libidn can be built into GNU Libc to enable a
@@ -522,6 +529,7 @@ stringprep_profile (const char *in,
  * \include example.c
  * \include example3.c
  * \include example4.c
+ * \include example5.c
  */
 
 /**
@@ -626,15 +634,6 @@ stringprep_profile (const char *in,
  * stringprep profile.  Returns 0 iff successful, or an error code.
  **/
 
-/**
- * stringprep_kerberos5:
- * @in: input/ouput array with string to prepare.
- * @maxlen: maximum length of input/output array.
- *
- * Prepare the input UTF-8 string according to the draft Kerberos5
- * stringprep profile.  Returns 0 iff successful, or an error code.
- **/
-
 /**
  * stringprep_plain:
  * @in: input/ouput array with string to prepare.
