@@ -30,6 +30,9 @@
 #include <stringprep.h>
 #include <punycode.h>
 #include <idna.h>
+#ifdef WITH_TLD
+# include <tld.h>
+#endif
 
 #include "idn_cmd.h"
 
@@ -85,8 +88,8 @@ main (int argc, char *argv[])
     {
       if (cmdn < args_info.inputs_num)
 	{
-	  strncpy(readbuf, args_info.inputs[cmdn++], BUFSIZ - 1);
-	  readbuf[BUFSIZ-1] = '\0';
+	  strncpy (readbuf, args_info.inputs[cmdn++], BUFSIZ - 1);
+	  readbuf[BUFSIZ - 1] = '\0';
 	}
       else if (fgets (readbuf, BUFSIZ, stdin) == NULL)
 	{
@@ -255,17 +258,17 @@ main (int argc, char *argv[])
 	    }
 
 	  q[len] = 0;
-	  p = stringprep_ucs4_to_utf8 (q, -1, NULL, NULL);
+	  r = stringprep_ucs4_to_utf8 (q, -1, NULL, NULL);
 	  free (q);
-	  if (!p)
+	  if (!r)
 	    {
 	      fprintf (stderr, "%s: could not convert from UCS-4 to UTF-8.\n",
 		       argv[0]);
 	      return 1;
 	    }
 
-	  r = stringprep_utf8_to_locale (p);
-	  free (p);
+	  p = stringprep_utf8_to_locale (r);
+	  free (r);
 	  if (!r)
 	    {
 	      fprintf (stderr, "%s: could not convert from UTF-8 to %s.\n",
@@ -273,9 +276,9 @@ main (int argc, char *argv[])
 	      return 1;
 	    }
 
-	  fprintf (stdout, "%s\n", r);
+	  fprintf (stdout, "%s\n", p);
 
-	  free (r);
+	  free (p);
 	}
 
       if (args_info.idna_to_ascii_given)
@@ -287,7 +290,6 @@ main (int argc, char *argv[])
 		       argv[0], stringprep_locale_charset ());
 	      return 1;
 	    }
-
 	  q = stringprep_utf8_to_ucs4 (p, -1, NULL);
 	  free (p);
 	  if (!q)
@@ -304,7 +306,7 @@ main (int argc, char *argv[])
 		fprintf (stderr, "input[%d] = U+%04x\n", i, q[i]);
 	    }
 
-	  rc = idna_to_ascii_4z (q, &r,
+	  rc = idna_to_ascii_4z (q, &p,
 				 (args_info.allow_unassigned_given ?
 				  IDNA_ALLOW_UNASSIGNED : 0) |
 				 (args_info.usestd3asciirules_given ?
@@ -312,21 +314,65 @@ main (int argc, char *argv[])
 	  free (q);
 	  if (rc != IDNA_SUCCESS)
 	    {
-	      fprintf (stderr, "%s: idna_to_ascii_from_locale() failed "
+	      fprintf (stderr, "%s: idna_to_ascii_4z() failed "
 		       "with error %d.\n", argv[0], rc);
 	      return 1;
 	    }
 
+#ifdef WITH_TLD
+	  if (args_info.tld_flag)
+	    {
+	      size_t errpos;
+
+	      rc = idna_to_unicode_8z4z (p, &q,
+					 (args_info.allow_unassigned_given ?
+					  IDNA_ALLOW_UNASSIGNED : 0) |
+					 (args_info.usestd3asciirules_given ?
+					  IDNA_USE_STD3_ASCII_RULES : 0));
+	      if (rc != IDNA_SUCCESS)
+		{
+		  fprintf (stderr, "%s: TLD idna_to_unicode_8z8z() failed "
+			   "with error %d.\n", argv[0], rc);
+		  return 1;
+		}
+
+	      if (args_info.debug_given)
+		{
+		  size_t i;
+		  for (i = 0; q[i]; i++)
+		    fprintf (stderr, "tld[%d] = U+%04x\n", i, q[i]);
+		}
+
+	      rc = tld_check_4z (q, &errpos, NULL);
+	      if (rc == TLD_INVALID)
+		{
+		  fprintf (stderr, "%s: string rejected by TLD test "
+			   "(Unicode position %d)\n", argv[0], errpos);
+		  free (q);
+		  return 1;
+		}
+	      if (rc != TLD_SUCCESS)
+		{
+		  fprintf (stderr, "%s: tld_check_4z failed with error %d.\n",
+			   argv[0], rc);
+		  free (q);
+		  return 1;
+		}
+
+	      free (r);
+	    }
+#endif
+
 	  if (args_info.debug_given)
 	    {
 	      size_t i;
-	      for (i = 0; r[i]; i++)
-		fprintf (stderr, "output[%d] = U+%04x\n", i, r[i]);
+	      for (i = 0; p[i]; i++)
+		fprintf (stderr, "output[%d] = U+%04x\n", i, p[i]);
 	    }
 
-	  fprintf (stdout, "%s\n", r);
+	  fprintf (stdout, "%s\n", p);
 
-	  free (r);
+	  free (p);
 	}
 
       if (args_info.idna_to_unicode_given)
@@ -364,7 +410,7 @@ main (int argc, char *argv[])
 	  free (p);
 	  if (rc != IDNA_SUCCESS)
 	    {
-	      fprintf (stderr, "%s: idna_to_unicode_locale_from_locale() "
+	      fprintf (stderr, "%s: idna_to_unicode_8z4z() "
 		       "failed with error %d.\n", argv[0], rc);
 	      return 1;
 	    }
@@ -376,17 +422,40 @@ main (int argc, char *argv[])
 		fprintf (stderr, "output[%d] = U+%04x\n", i, q[i]);
 	    }
 
-	  p = stringprep_ucs4_to_utf8 (q, -1, NULL, NULL);
+#ifdef WITH_TLD
+	  if (args_info.tld_flag)
+	    {
+	      size_t errpos;
+
+	      rc = tld_check_4z (q, &errpos, NULL);
+	      if (rc == TLD_INVALID)
+		{
+		  fprintf (stderr, "%s: string rejected by TLD test "
+			   "(Unicode position %d)\n", argv[0], errpos);
+		  free (q);
+		  return 1;
+		}
+	      if (rc != TLD_SUCCESS)
+		{
+		  fprintf (stderr, "%s: tld_check_4z failed with error %d.\n",
+			   argv[0], rc);
+		  free (q);
+		  return 1;
+		}
+	    }
+#endif
+
+	  r = stringprep_ucs4_to_utf8 (q, -1, NULL, NULL);
 	  free (q);
-	  if (!p)
+	  if (!r)
 	    {
 	      fprintf (stderr, "%s: could not convert from UTF-8 to UCS-4.\n",
 		       argv[0]);
 	      return 1;
 	    }
 
-	  r = stringprep_utf8_to_locale (p);
-	  free (p);
+	  p = stringprep_utf8_to_locale (r);
+	  free (r);
 	  if (!r)
 	    {
 	      fprintf (stderr, "%s: could not convert from UTF-8 to %s.\n",
@@ -394,11 +463,10 @@ main (int argc, char *argv[])
 	      return 1;
 	    }
 
-	  fprintf (stdout, "%s\n", r);
+	  fprintf (stdout, "%s\n", p);
 
-	  free (r);
+	  free (p);
 	}
-
     }
   while (!feof (stdin) && !ferror (stdin) && (args_info.inputs_num == 0 ||
 					      cmdn < args_info.inputs_num));
